@@ -1,4 +1,4 @@
-/*
+﻿/*
  *
  *  Copyright ( c ) 2011-2015
  *  name : Francis Banyikwa
@@ -37,6 +37,31 @@ class favorites : public QDialog
 {
 	Q_OBJECT
 public:
+	template< typename E,typename F >
+	static void _stringListToStrings( const F& s,size_t n,size_t k,E& e )
+	{
+		if( n < k ){
+
+			e = s.at( n ) ;
+		}
+	}
+
+	template< typename E,typename F,typename ... T >
+	static void _stringListToStrings( const E& s,size_t n,size_t k,F& e,T& ... t )
+	{
+		if( n < k ){
+
+			e = s.at( n ) ;
+			_stringListToStrings( s,n + 1,k,t ... ) ;
+		}
+	}
+
+	template< typename E,typename ... F >
+	static void stringListToStrings( const E& s,F& ... t )
+	{
+		_stringListToStrings( s,0,size_t( s.size() ),t ... ) ;
+	}
+
 	struct entry
 	{
 		entry()
@@ -63,7 +88,8 @@ public:
 					 mountPointPath,
 					 autoMountVolume,
 					 configFilePath,
-					 idleTimeOut } ;
+					 idleTimeOut,
+					 mountOptions } ;
 			}
 		}
 
@@ -84,12 +110,16 @@ public:
 				}
 			} ;
 
-			auto e = "%1\t%2\t%3\t%4\t%5\t" ;
+			auto e = "%1\t%2\t%3\t%4\t%5\t%6\t" ;
 
 			return QString( e ).arg( volumePath,mountPointPath,autoMountVolume,
-						 _opt( configFilePath ),_opt( idleTimeOut ) ) ;
+						 _opt( configFilePath ),_opt( idleTimeOut ),
+						 _opt( mountOptions ) ) ;
+		}
+		QStringList configStringList() const
+		{
+			return this->configString().split( "\t",QString::SkipEmptyParts ) ;
 		}
-
 		bool operator!=( const favorites::entry& other ) const
 		{
 			return !( *this == other ) ;
@@ -101,7 +131,8 @@ public:
 				this->mountPointPath  == other.mountPointPath &&
 				this->autoMountVolume == other.autoMountVolume &&
 				this->configFilePath  == other.configFilePath &&
-				this->idleTimeOut     == other.idleTimeOut ;
+				this->idleTimeOut     == other.idleTimeOut &&
+				this->mountOptions    == other.mountOptions ;
 		}
 
 		bool autoMount() const
@@ -114,27 +145,32 @@ public:
 		QString autoMountVolume ;
 		QString configFilePath ;
 		QString idleTimeOut ;
+		QString mountOptions ;
 
 	private:
 		void config( const QStringList& e )
 		{
-			if( e.size() > 4 ){
+			favorites::stringListToStrings( e,
+							volumePath,
+							mountPointPath,
+							autoMountVolume,
+							configFilePath,
+							idleTimeOut,
+							mountOptions ) ;
 
-				volumePath      = e.at( 0 ) ;
-				mountPointPath  = e.at( 1 ) ;
-				autoMountVolume = e.at( 2 ) ;
-				configFilePath  = e.at( 3 ) ;
-				idleTimeOut     = e.at( 4 ) ;
+			if( configFilePath == "N/A" ){
 
-				if( configFilePath == "N/A" ){
+				configFilePath.clear() ;
+			}
 
-					configFilePath.clear() ;
-				}
+			if( idleTimeOut == "N/A" ){
 
-				if( idleTimeOut == "N/A" ){
+				idleTimeOut.clear() ;
+			}
 
-					idleTimeOut.clear() ;
-				}
+			if( mountOptions == "N/A" ){
+
+				mountOptions.clear() ;
 			}
 		}
 	};
@@ -164,6 +200,7 @@ private slots:
 	void shortcutPressed( void ) ;
 	void devicePathTextChange( QString ) ;
 private:
+	void checkFavoritesConsistency() ;
 	favorites::entry getEntry( int ) ;
 	QString getExistingFile( const QString& ) ;
 	QString getExistingDirectory( const QString& ) ;
@@ -171,6 +208,7 @@ private:
 	bool eventFilter( QObject * watched,QEvent * event ) ;
 	void addEntries( const QStringList& ) ;
 	Ui::favorites * m_ui ;
+	QWidget * m_parentWidget ;
 };
 
 #endif // MANAGEDEVICENAMES_H
