@@ -31,7 +31,8 @@ options::options( QWidget * parent,bool r,const QStringList& l,
 	m_ui( new Ui::options ),
 	m_create( r ),
 	m_type( l.at( 2 ).toLower() ),
-	m_setOptions( std::move( e ) )
+	m_setOptions( std::move( e ) ),
+	m_parentWidget( parent )
 {
 	m_ui->setupUi( this ) ;
 
@@ -50,7 +51,12 @@ options::options( QWidget * parent,bool r,const QStringList& l,
 	m_ui->lineEditIdleTime->setText( l.at( 0 ) ) ;
 	m_ui->lineConfigFilePath->setText( l.at( 1 ) ) ;
 
+	utility::setWindowOptions( this ) ;
+	utility::setParent( parent,&m_parentWidget,this ) ;
+
 	this->show() ;
+	this->raise() ;
+	this->activateWindow() ;
 }
 
 void options::pushButton()
@@ -136,7 +142,7 @@ void options::pbOK()
 
 	if( !ok ){
 
-		DialogMsg( this ).ShowUIOK( tr( "ERROR" ),tr( "Idle Time Field Requires Digits Only If Not Empty." ) ) ;
+		DialogMsg( m_parentWidget,this ).ShowUIOK( tr( "ERROR" ),tr( "Idle Time Field Requires Digits Only If Not Empty." ) ) ;
 	}else{
 		m_setOptions( { e,m_ui->lineConfigFilePath->text() } ) ;
 
